function [C,P]=bsvalue(S,K,T,r,v)
%BSVALUE The Black-Scholes (1973) Model for evaluating European options on 
%	non-dividend paying stocks. For stocks which pay dividends, the current
%	stock price S can be replaced by the current price less the present
%	value of the expected dividend: S-d*exp(-rt),where d is the expected 
%	dividend payout and t is the time to payout.
%
%	Syntax: [C,P] = BSVALUE(S,K,T,r,v)
%
%	S	- the stock price
%	K	- the strike price
%	T	- the time to expiration in days
%	r	- the risk free interest rate in percent
%	v	- the annual volatility in percent
%
%	C	- the theoretical value of a call
%	P	- the theoretical value of a put

% Copyright (c) 1996-98 by Rafal Weron.
% Written by: R. Weron.

T=T/365;
e_rt=exp(-0.01*r.*T);
vt=0.01*v.*sqrt(T);
h=(log(S./(K.*e_rt))./vt)+vt/2;

C=S.*normcdf(h)-K.*e_rt.*normcdf(h-vt);
P=-S.*normcdf(-h)+K.*e_rt.*normcdf(vt-h);
