function greek3(p,r,v,type,minp,maxp,nump,numt)
%GREEK3 Internally used by PART2.

% Copyright (c) 1996-98 by Rafal Weron.
% Written by: J. Glowinski, R. Weron.

nocontracts=p(:,1);
shortlong=p(:,2);
strike=p(:,3);
contract=p(:,4);
maturity=p(:,5);

if nargin<=6,
  nump=40;
  numt=10;
  if nargin<=4,
    minstrike=min(strike);
    maxstrike=max(strike);
    difstrike=maxstrike-minstrike;
    if difstrike==0, difstrike=round(1+maxstrike/100); end;
    minp=minstrike-difstrike;
    maxp=maxstrike+difstrike;
  end
end;

options=(contract~=0);
U=linspace(minp,maxp,nump);
if sum(options)>0
  maturity2=maturity(options);
  mint=min(maturity2);
  maxt=max(maturity2);
else
  mint=min(maturity);
  maxt=max(maturity);
end
time=linspace(0,mint,numt);
numt=numt-1;
time=time(1:numt);
size1=size(p,1);
r=r/100;
v=v/100;
type=upper(type);

greek=zeros(numt,nump);
if strcmp(type,'DELTA')
  for i=1:numt,
    temp=zeros(1,nump);
    for j=1:size1,
      if contract(j)==0
        if shortlong(j)==1
          temp=temp-nocontracts(j);
        else
          temp=temp+nocontracts(j);
        end
      else
        K=strike(j);
        T=(maturity(j)-time(i))/365;
        e_rt=exp(-r.*T);
        vt=v.*sqrt(T);
        h=(log(U./K)./vt)+vt/2;
        if contract(j)==1
          delta=e_rt.*normcdf(h);
        else
          delta=-e_rt.*normcdf(-h);
        end
        if shortlong(j)==1
          temp=temp-nocontracts(j)*delta;
        else
          temp=temp+nocontracts(j)*delta;
        end
      end
    end
    greek(i,:)=temp;
  end
elseif strcmp(type,'GAMMA')
  for i=1:numt,
    temp=zeros(1,nump);
    for j=1:size1,
      if contract(j)~=0
        K=strike(j);
        T=(maturity(j)-time(i))/365;
        e_rt=exp(-r.*T);
        vt=v.*sqrt(T);
        h=(log(U./K)./vt)+vt/2;
        gamma=(e_rt.*normpdf(h))./(U.*vt);
        if shortlong(j)==1
          temp=temp-nocontracts(j)*gamma;
        else
          temp=temp+nocontracts(j)*gamma;
        end
      end
    end
    greek(i,:)=temp;
  end
elseif strcmp(type,'THETA')
  for i=1:numt,
    temp=zeros(1,nump);
    for j=1:size1,
      if contract(j)~=0
        K=strike(j);
        T=(maturity(j)-time(i))/365;
        e_rt=exp(-r.*T);
        vt=v.*sqrt(T);
        h=(log(U./K)./vt)+vt/2;
        if contract(j)==1
          theta=(v.*U.*e_rt.*normpdf(h)/2)./sqrt(T)+r.*K.*e_rt.*...
                normcdf(h-vt)-r.*U.*e_rt.*normcdf(h);
        else
          theta=(v.*U.*e_rt.*normpdf(h)/2)./sqrt(T)-r.*K.*e_rt.*...
                normcdf(vt-h)+r.*U.*e_rt.*normcdf(-h);
        end;
        if shortlong(j)==1
          temp=temp-nocontracts(j)*theta;
        else
          temp=temp+nocontracts(j)*theta;
        end
      end
    end
    greek(i,:)=temp;
  end
elseif strcmp(type,'VEGA')
  for i=1:numt,
    temp=zeros(1,nump);
    for j=1:size1,
      if contract(j)~=0
        K=strike(j);
        T=(maturity(j)-time(i))/365;
        e_rt=exp(-r.*T);
        vt=v.*sqrt(T);
        h=(log(U./K)./vt)+vt/2;
        vega=e_rt.*U.*sqrt(T).*normpdf(h);
        if shortlong(j)==1
          temp=temp-nocontracts(j)*vega;
        else
          temp=temp+nocontracts(j)*vega;
        end
      end
    end
    greek(i,:)=temp;
  end
elseif strcmp(type,'RHO')
  for i=1:numt,
    temp=zeros(1,nump);
    for j=1:size1,
      if contract(j)~=0
        K=strike(j);
        T=(maturity(j)-time(i))/365;
        e_rt=exp(-r.*T);
        vt=v.*sqrt(T);
        h=(log(U./K)./vt)+vt/2;
        if contract(j)==1
          rho=-T.*(U.*e_rt.*normcdf(h)-K.*e_rt.*normcdf(h-vt));
        else
          rho=-T.*(-U.*e_rt.*normcdf(-h)+K.*e_rt.*normcdf(vt-h));
        end
        if shortlong(j)==1
          temp=temp-nocontracts(j)*rho;
        else
          temp=temp+nocontracts(j)*rho;
        end
      end
    end
    greek(i,:)=temp;
  end
else
  disp('Unknown indicator type');
  return;
end

mesh2(U,maxt-time,greek);
xlabel('Underlying');
ylabel('Maturity');
zlabel(stdstr(type));
